/* alpha.cpp
 * Copyright (c) 1993-1997, Silicon Graphics, Inc.
 * 
 * alphaMouse.cpp
 * Modified by Petr Felkel, 2007-8, felkel@fel.cvut.cz, for teaching fundamentals of graphics
 *		- four blending modes and alpha set by mouse
 *		- printed help on Console
 *
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

/*
 *  alpha.c
 *  This program draws several overlapping filled polygons
 *  to demonstrate the effect order has on alpha blending results.
 *  Use the 't' key to toggle the order of drawing polygons.
 */
#include <stdlib.h>
#include <GL/glut.h>
#include <stdio.h>  // for vsprintf
#include <stdarg.h> // for argument parsing of strokeOutput()

static int leftFirst = GL_TRUE;  // triangle order
static float leftAlpha = 0.5;    // alpha for left triangle
static float rightAlpha = 0.5;   //alpha for right triangle
static float lastX, lastY;       
static int win_width, win_height;  // viewport size
static int changeAlpha = GL_FALSE;

/* Text output routines */
//bitmapOutputFormated(30, 240,  GLUT_BITMAP_HELVETICA_18, "%d, Helvetica %f is yet another bitmap font.", 1, 6.667); 
void bitmapOutputFormated(int x, int y, void *font, char *format,...) {
	va_list args;
	char buffer[200], *p;

	va_start(args, format);
	vsprintf(buffer, format, args);
	va_end(args);

	glRasterPos2f(x, y);  
	for (p = buffer; *p; p++)
		glutBitmapCharacter(font, *p);
} 
 void strokeOutput(GLfloat x, GLfloat y, char *format,...) {
 va_list args;
 char buffer[200], *p;

  va_start(args, format);
  vsprintf(buffer, format, args);
  va_end(args);
  glPushMatrix();
  glTranslatef(x, y, 0);
  glScalef(0.0004, 0.0004, 0.0004);
  for (p = buffer; *p; p++)
    glutStrokeCharacter(GLUT_STROKE_ROMAN, *p);
  glPopMatrix();
} 

/*  Initialize alpha blending function.
 */
static void init(void)
{
   glEnable (GL_BLEND);
   glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glShadeModel (GL_FLAT);
   glClearColor (0.0, 0.0, 0.0, 0.0);
}

static void drawLeftTriangle(void)
{
   /* draw yellow triangle on LHS of screen */
   
   glBegin (GL_TRIANGLES);
      glColor4f(1.0, 1.0, 0.0, leftAlpha);
      glVertex3f(0.7, 0.5, 0.0); 
      glVertex3f(0.1, 0.9, 0.0); 
      glVertex3f(0.1, 0.1, 0.0); 
   glEnd();
}

static void drawRightTriangle(void)
{
   /* draw cyan triangle on RHS of screen */

   glBegin (GL_TRIANGLES);
      glColor4f(0.0, 1.0, 1.0, rightAlpha);
      glVertex3f(0.9, 0.9, 0.0); 
      glVertex3f(0.3, 0.5, 0.0); 
      glVertex3f(0.9, 0.1, 0.0); 
   glEnd();
}

void display(void)
{
   glClear(GL_COLOR_BUFFER_BIT);

   if (leftFirst) {
      drawLeftTriangle();
	  strokeOutput( 0.25, 0.85, "First", leftAlpha );
      drawRightTriangle();

   }
   else {
      drawRightTriangle();
	  strokeOutput( 0.65, 0.85, "First", leftAlpha );
      drawLeftTriangle();
   }
	
   glColor4f( 1.0, 1.0, 1.0, 1.0 );
   strokeOutput( 0.15, 0.95, "A = %3.2f", leftAlpha );
   strokeOutput( 0.6, 0.95, "A = %3.2f", rightAlpha );
   glutSwapBuffers();
}

void reshape(int w, int h)
{
   win_width = w;
   win_height = h;
   glViewport(0, 0, (GLsizei) w, (GLsizei) h);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (w <= h) 
      gluOrtho2D (0.0, 1.0, 0.0, 1.0*(GLfloat)h/(GLfloat)w);
   else 
      gluOrtho2D (0.0, 1.0*(GLfloat)w/(GLfloat)h, 0.0, 1.0);
}

void printHelp( void )
{
	printf("\n");

	printf("Mouse drag in left part - left triangle alpha\n");
	printf("Mouse drag in right part - right triangle alpha\n");
	printf("T - Switch the order of triangles\n");
	printf("1 - glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)\n");
	printf("2 - glBlendFunc (GL_SRC_ALPHA, GL_ONE)\n");
	printf("3 - glBlendFunc (GL_SRC_ALPHA, GL_ZERO)\n");
	printf("4 - glBlendFunc (GL_ONE, GL_ZERO)\n");
}
void keyboard(unsigned char key, int x, int y)
{
   switch (key) {
	  case 'h':
		  printHelp();
		  break;
      case 't':
      case 'T':
         leftFirst = !leftFirst;
		 //printf( "%s \n", leftFirst ? "leftFirst" : "rightFirst" );
	
         break;
      case 27:  /*  Escape key  */
         exit(0);
         break;
	  case '1':
		     glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			 printf("1 - glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)\n");
			 break;
	  case '2':
		     glBlendFunc (GL_SRC_ALPHA, GL_ONE);
			 printf("2 - glBlendFunc (GL_SRC_ALPHA, GL_ONE)\n");
			 break;
	  case '3':
		     glBlendFunc (GL_SRC_ALPHA, GL_ZERO);
			 printf("3 - glBlendFunc (GL_SRC_ALPHA, GL_ZERO)\n");
			 break;
	  case '4':
		     glBlendFunc (GL_ONE, GL_ZERO);
			 printf("4 - glBlendFunc (GL_ONE, GL_ZERO)\n");
			 break;
      default:
         break;
   }
   glutPostRedisplay();
}

void mouse( int button, int state, int x, int y )
{
	lastX = x;
	lastY = y;
   	if( button == GLUT_LEFT_BUTTON ) 
	{
	  if( state == GLUT_DOWN )
	  {
		 changeAlpha = GL_TRUE;
	  }
	  else
	  {
		 changeAlpha = GL_FALSE;
	  }
	}
}
void motion( int x, int y )
{	
	if( lastX < win_width / 2 ) 
	{
		leftAlpha = 1.0 - (float) y / win_height;
		if( leftAlpha < 0.0 ) 
			leftAlpha = 0.0;
		else if( leftAlpha > 1.0 )
			leftAlpha = 1.0;

	}
	else 
	{
		rightAlpha = 1.0 - (float) y / win_height;
		if( rightAlpha < 0.0 ) 
			rightAlpha = 0.0;
		else if( rightAlpha > 1.0 )
			rightAlpha = 1.0;
	}
	glutPostRedisplay();
}

/*  Main Loop
 *  Open window with initial window size, title bar, 
 *  RGBA display mode, and handle input events.
 */
int main(int argc, char** argv)
{
   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);
   glutInitWindowSize (300, 300);
   glutCreateWindow ("alphaMouse  mousedrag alpha, t toggle order, h help, 1,2,3,4 blendFunctions");
   init();
   glutReshapeFunc (reshape);
   glutKeyboardFunc (keyboard);
   glutMouseFunc( mouse );
   glutMotionFunc( motion );
   glutDisplayFunc (display);
   printHelp();
   glutMainLoop();
   return 0;
} 
